/*
    DoubleSimple: DoubleStringEpsilon.m
    Copyright (C) 2003-2006  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleStringEpsilon class

*/

#import <DoubleStringEpsilon.h>
#import "purify.h"

#define OPERATOR(method, arg) \
  id retval = [super method: arg]; \
  d = purify_string_epsilon(d, [self getSignificantFigures], \
                            [self getEpsilon]); \
  return retval

@implementation DoubleStringEpsilon

/* Methods to override from DoubleSimple */

-set: (DoubleSimple *)value {
  OPERATOR(set, value);
}

-cset: (double)value {
  OPERATOR(cset, value);
}

-setStr: (char *)str {
  OPERATOR(setStr, str);
}

-read: (FILE *)fp {
  OPERATOR(read, fp);
}


-add: (DoubleSimple *)value {
  OPERATOR(add, value);
}

-sub: (DoubleSimple *)value {
  OPERATOR(sub, value);
}

-mul: (DoubleSimple *)value {
  OPERATOR(mul, value);
}

-div: (DoubleSimple *)value {
  OPERATOR(div, value);
}


-iadd: (int)value {
  OPERATOR(iadd, value);
}

-isub: (int)value {
  OPERATOR(isub, value);
}

-imul: (int)value {
  OPERATOR(imul, value);
}

-idiv: (int)value {
  OPERATOR(idiv, value);
}


-cadd: (double)value {
  OPERATOR(cadd, value);
}

-csub: (double)value {
  OPERATOR(csub, value);
}

-cmul: (double)value {
  OPERATOR(cmul, value);
}

-cdiv: (double)value {
  OPERATOR(cdiv, value);
}

@end
